/*
* Copyright 2003(c) Theo Pavlidis
* See PixEx.h for a detailed notice on the use of the software
*/

//	Quantizations Functions used in p4pApp
	
#include "stdafx.h"
#include <math.h>
#include "quant.h"

#define LVL 256

void MostSignBits(
	unsigned char *InputImg,
	int width, int height, int bpp,	// dimensions of image and pixels
	int zones[],	// how many partitions
	unsigned char *OutputImg
	)
{
	int x, y, j, k;
	unsigned char mask[3] = { 0340, 0340, 0340 };	// default: 3 most signifiacant bits
	unsigned char bmask[8] = {0200, 0300, 0340, 0360, 0370, 0374, 0376, 0377 };	// to handle 0 bits!
	int nb;
	double lg2, b2 = log(2.0);
	for(k=0; k<bpp; k++) {
		if(zones[k]>0) {
			lg2 = log((double)zones[k])/b2;
			nb = (int)ceil(lg2);
			if(nb>8) nb=8;	else if(nb<1) nb=1;
			mask[k] = bmask[nb-1];
		}
	}
	for(y=0, j=0; y<height; y++) {
		for(x=0; x<width; x++, j += bpp) {
			for(k=0; k<bpp; k++) OutputImg[j+k] = (InputImg[j+k])&mask[k];
		}
	}
}

//	Histogram equalization per Algorithm 3.2 of GRIP modified by assuming
//	that new levels are much less than LVL, so there is no ned to split intensities
void HistEqualization(
	unsigned char *InputImg,
	int width, int height, int bpp,	// dimensions of image and pixels
	int zones[],	// how many partitions
	unsigned char *OutputImg
	)
{
	int x, y, j, k;
	long h[LVL];
	double total, havg;
	int nz, rz, vmap[LVL];
	double factor = 32;

	for(k=0; k<bpp; k++) {	// for each color
		// compute histogram
		for(j=0; j<LVL; j++) h[j] = 0;
		for(y=0, j=0; y<height; y++) {
			for(x=0; x<width; x++, j += bpp) h[InputImg[j+k]]++;
		}
		for(j=0, total=0; j<LVL; j++) total += h[j];	// integral of histogram
		// equalize histogram
		if(zones[k]>0) {
			nz = zones[k];
			factor = 256.0/nz;
		}
		else nz = 8;
		havg = total/nz;
		for(j=0, total=0, rz=0; j<LVL; j++) {
			vmap[j] = (int)(factor*rz + 0.5);
			total += h[j];
			while(total > havg) {
				total -= havg;	rz++;
			}
		}
		// transform image
		for(y=0, j=0; y<height; y++) {
			for(x=0; x<width; x++, j += bpp) OutputImg[j+k] = vmap[InputImg[j+k]];
		}
	}
}